==> v6.8.tar.gz: wcs/tests/admin_pages/test_all.py <==
# -*- coding: utf-8 -*-

import os

import pytest

from wcs.qommon.ident.password_accounts import PasswordAccount
from wcs.qommon.http_request import HTTPRequest
from wcs.roles import Role
from wcs.workflows import Workflow, ChoiceWorkflowStatusItem, JumpOnSubmitWorkflowStatusItem
from wcs.wf.redirect_to_url import RedirectToUrlWorkflowStatusItem
from wcs.wf.create_formdata import CreateFormdataWorkflowStatusItem, Mapping
from wcs.formdef import FormDef
from wcs import fields

from utilities import get_app, login, create_temporary_pub, clean_temporary_pub


def pytest_generate_tests(metafunc):
    if 'pub' in metafunc.fixturenames:
        metafunc.parametrize('pub', ['pickle', 'sql', 'pickle-templates'], indirect=True)


@pytest.fixture
def pub(request):
    pub = create_temporary_pub(
        sql_mode=bool('sql' in request.param), templates_mode=bool('templates' in request.param)
    )

    req = HTTPRequest(None, {'SCRIPT_NAME': '/', 'SERVER_NAME': 'example.net'})
    pub.set_app_dir(req)
    pub.cfg['identification'] = {'methods': ['password']}
    pub.cfg['language'] = {'language': 'en'}
    pub.write_cfg()

    return pub


def create_superuser(pub):
    if pub.user_class.select(lambda x: x.name == 'admin'):
        user1 = pub.user_class.select(lambda x: x.name == 'admin')[0]
        user1.is_admin = True
        user1.store()
        return user1

    user1 = pub.user_class(name='admin')
    user1.is_admin = True
    user1.store()

    account1 = PasswordAccount(id='admin')
    account1.set_password('admin')
    account1.user_id = user1.id
    account1.store()

    return user1


def create_role():
    Role.wipe()
    role = Role(name='foobar')
    role.store()
    return role


def teardown_module(module):
    clean_temporary_pub()


def test_empty_site(pub):
    resp = get_app(pub).get('/backoffice/')
    resp = resp.click('Users', index=0)
    resp = resp.click('New User')
    resp = get_app(pub).get('/backoffice/')
    resp = resp.click('Settings', index=0)


def test_empty_site_but_idp_settings(pub):
    pub.cfg['idp'] = {'xxx': {}}
    pub.write_cfg()
    resp = get_app(pub).get('/backoffice/')
    assert resp.location == 'http://example.net/login/?next=http%3A%2F%2Fexample.net%2Fbackoffice%2F'


def test_with_user(pub):
    create_superuser(pub)
    resp = get_app(pub).get('/backoffice/', status=302)
    assert resp.location == 'http://example.net/login/?next=http%3A%2F%2Fexample.net%2Fbackoffice%2F'


def test_with_superuser(pub):
    create_superuser(pub)
    app = login(get_app(pub))
    app.get('/backoffice/')


def test_admin_redirect(pub):
    create_superuser(pub)
    app = login(get_app(pub))
    assert app.get('/admin/whatever', status=302).location == 'http://example.net/backoffice/whatever'


def test_admin_for_all(pub):
    user = create_superuser(pub)
    role = create_role()

    try:
        open(os.path.join(pub.app_dir, 'ADMIN_FOR_ALL'), 'w').close()
        resp = get_app(pub).get('/backoffice/', status=200)
        # check there is a CSS class
        assert resp.pyquery.find('body.admin-for-all')
        # check there are menu items
        resp.click('Management', index=0)
        resp.click('Forms', index=0)
        resp.click('Settings', index=0)

        # cheeck it's possible to get inside the subdirectories
        resp = get_app(pub).get('/backoffice/settings/', status=200)

        pub.cfg['admin-permissions'] = {'settings': [role.id]}
        pub.write_cfg()
        resp = get_app(pub).get('/backoffice/settings/', status=200)

        # check it doesn't work with a non-empty ADMIN_FOR_ALL file
        fd = open(os.path.join(pub.app_dir, 'ADMIN_FOR_ALL'), 'w')
        fd.write('x.x.x.x')
        fd.close()
        resp = get_app(pub).get('/backoffice/settings/', status=302)

        # check it works if the file contains the user IP address
        fd = open(os.path.join(pub.app_dir, 'ADMIN_FOR_ALL'), 'w')
        fd.write('127.0.0.1')
        fd.close()
        resp = get_app(pub).get('/backoffice/settings/', status=200)

        # check it's also ok if the user is logged in but doesn't have the
        # permissions
        user.is_admin = False
        user.store()
        resp = login(get_app(pub)).get('/backoffice/settings/', status=200)
        # check there are menu items
        resp.click('Management', index=0)
        resp.click('Forms', index=0)
        resp.click('Settings', index=0)

    finally:
        if 'admin-permissions' in pub.cfg:
            del pub.cfg['admin-permissions']
            pub.write_cfg()
        os.unlink(os.path.join(pub.app_dir, 'ADMIN_FOR_ALL'))
        role.remove_self()
        user.is_admin = True
        user.store()


def test_users_roles_menu_entries(pub):
    create_superuser(pub)
    app = login(get_app(pub))
    resp = app.get('/backoffice/')
    assert 'Users' in resp.text
    assert 'Roles' in resp.text
    resp = app.get('/backoffice/menu.json')
    assert 'Users' in [x['label'] for x in resp.json]
    assert 'Roles' in [x['label'] for x in resp.json]

    # don't include users/roles in menu if roles are managed by an external
    # identity provider.
    pub.cfg['sp'] = {'idp-manage-roles': True}
    pub.write_cfg()

    resp = app.get('/backoffice/')
    assert 'Users' not in resp.text
    assert 'Roles' not in resp.text
    resp = app.get('/backoffice/menu.json')
    assert 'Users' not in [x['label'] for x in resp.json]
    assert 'Roles' not in [x['label'] for x in resp.json]


def test_studio_home(pub):
    create_superuser(pub)
    app = login(get_app(pub))
    resp = app.get('/backoffice/')
    assert 'studio' in resp.text
    resp = app.get('/backoffice/studio/')
    assert '../forms/' in resp.text
    assert '../cards/' in resp.text
    assert '../workflows/' in resp.text

    pub.cfg['admin-permissions'] = {}
    for part in ('forms', 'cards', 'workflows'):
        # check section link are not displayed if user has no access right
        pub.cfg['admin-permissions'].update({part: ['x']})  #  block access
        pub.write_cfg()
        if part != 'workflows':
            resp = app.get('/backoffice/studio/')
            assert '../%s/' % part not in resp.text
        else:
            resp = app.get('/backoffice/studio/', status=403)  # totally closed

    resp = app.get('/backoffice/')
    assert 'studio' not in resp.text


def test_studio_workflows(pub):
    create_superuser(pub)
    app = login(get_app(pub))
    resp = app.get('/backoffice/workflows/')
    resp = resp.click(r'Default \(cards\)')
    assert 'status/recorded/' in resp.text
    assert 'status/deleted/' in resp.text
    assert 'This is the default workflow,' in resp.text


def test_create_formdata(pub):
    create_superuser(pub)
    create_role()

    FormDef.wipe()
    target_formdef = FormDef()
    target_formdef.name = 'target form'
    target_formdef.enable_tracking_codes = True
    target_formdef.fields = [
        fields.StringField(id='0', label='string', varname='foo_string'),
        fields.FileField(id='1', label='file', type='file', varname='foo_file'),
    ]
    target_formdef.store()

    Workflow.wipe()
    wf = Workflow(name='create-formdata')

    st1 = wf.add_status('New')
    st2 = wf.add_status('Resubmit')

    jump = ChoiceWorkflowStatusItem()
    jump.id = '_resubmit'
    jump.label = 'Resubmit'
    jump.by = ['_submitter']
    jump.status = st2.id
    jump.parent = st1
    st1.items.append(jump)

    create_formdata = CreateFormdataWorkflowStatusItem()
    create_formdata.id = '_create_formdata'
    create_formdata.formdef_slug = target_formdef.url_name
    create_formdata.mappings = [
        Mapping(field_id='0', expression='=form_var_toto_string'),
        Mapping(field_id='1', expression='=form_var_toto_file_raw'),
    ]
    create_formdata.parent = st2
    st2.items.append(create_formdata)

    redirect = RedirectToUrlWorkflowStatusItem()
    redirect.id = '_redirect'
    redirect.url = '{{ form_links_resubmitted.form_url }}'
    redirect.parent = st2
    st2.items.append(redirect)

    jump = JumpOnSubmitWorkflowStatusItem()
    jump.id = '_jump'
    jump.status = st1.id
    jump.parent = st2
    st2.items.append(jump)

    wf.store()

    app = login(get_app(pub))
    resp = app.get('/backoffice/workflows/%s/status/%s/' % (wf.id, st2.id))
    pq = resp.pyquery.remove_namespaces()
    assert pq('option[value="New Form Creation"]').text() == 'New Form Creation'
    assert pq('#itemId__create_formdata a')[0].text == 'New Form Creation (target form)'

    resp = resp.click(
        'Edit',
        href='items/_create_formdata/',
    )
    resp.form.set('varname', 'resubmitted')
    resp = resp.form.submit(name='submit')
    resp = resp.follow()

    # checks that nothing changed after submit
    wf2 = Workflow.select()[0]
    item = wf2.get_status('2').items[0]
    assert item.varname == 'resubmitted'
    assert isinstance(item, CreateFormdataWorkflowStatusItem)
    wf.get_status('2').items[0].label = 'really resubmit'

    # duplicate
    resp = app.get('/backoffice/workflows/%s/status/%s/items/_create_formdata/' % (wf.id, st2.id))
    resp.form.set('mappings$element1$field_id', '0')
    resp = resp.form.submit(name='submit')
    pq = resp.pyquery.remove_namespaces()
    assert pq('.error').text() == 'Some destination fields are duplicated'

    # check setting map_fields_by_varname on new action
    resp = app.get('/backoffice/workflows/%s/status/%s/' % (wf.id, st2.id))
    resp.forms['new-action-form']['action-formdata-action'] = 'New Form Creation'
    resp = resp.forms['new-action-form'].submit()
    resp = resp.follow()
    resp = resp.click(r'New Form Creation \(not configured\)')
    resp.form['formdef_slug'] = 'target-form'  # set target form
    resp = resp.form.submit('submit')
    assert 'Please define new mappings' in resp
    assert resp.form['map_fields_by_varname'].checked is False
    resp.form['map_fields_by_varname'].checked = True
    resp = resp.form.submit('submit')
    resp = resp.follow()
    wf = Workflow.get(wf.id)
    st2 = wf.possible_status[-1]
    assert wf.possible_status[-1].items[-1].map_fields_by_varname is True
    assert wf.possible_status[-1].items[-1].mappings is None
    assert wf.possible_status[-1].items[-1].formdef_slug == 'target-form'
    resp = app.get('/backoffice/workflows/%s/status/%s/items/%s/' % (wf.id, st2.id, st2.items[-1].id))
    resp.form['formdef_slug'] = ''  # unset target form
    resp.form.submit('submit')
    wf = Workflow.get(wf.id)
    assert wf.possible_status[-1].items[-1].formdef_slug is None
    assert wf.possible_status[-1].items[-1].map_fields_by_varname is True
    resp = app.get('/backoffice/workflows/%s/status/%s/items/%s/' % (wf.id, st2.id, st2.items[-1].id))
    resp.form['formdef_slug'] = 'target-form'  # reset target form
    resp.form.submit('submit')  # no error

==> v6.30.tar.gz: wcs/tests/admin_pages/test_all.py <==
# -*- coding: utf-8 -*-

import os

import pytest
from utilities import clean_temporary_pub, create_temporary_pub, get_app, login

from wcs.qommon.http_request import HTTPRequest
from wcs.qommon.ident.password_accounts import PasswordAccount


def pytest_generate_tests(metafunc):
    if 'pub' in metafunc.fixturenames:
        metafunc.parametrize('pub', ['pickle', 'sql', 'pickle-templates'], indirect=True)


@pytest.fixture
def pub(request):
    pub = create_temporary_pub(
        sql_mode=bool('sql' in request.param), templates_mode=bool('templates' in request.param)
    )

    req = HTTPRequest(None, {'SCRIPT_NAME': '/', 'SERVER_NAME': 'example.net'})
    pub.set_app_dir(req)
    pub.cfg['identification'] = {'methods': ['password']}
    pub.cfg['language'] = {'language': 'en'}
    pub.write_cfg()

    return pub


def create_superuser(pub):
    if pub.user_class.select(lambda x: x.name == 'admin'):
        user1 = pub.user_class.select(lambda x: x.name == 'admin')[0]
        user1.is_admin = True
        user1.store()
        return user1

    user1 = pub.user_class(name='admin')
    user1.is_admin = True
    user1.store()

    account1 = PasswordAccount(id='admin')
    account1.set_password('admin')
    account1.user_id = user1.id
    account1.store()

    return user1


def create_role(pub):
    pub.role_class.wipe()
    role = pub.role_class(name='foobar')
    role.store()
    return role


def teardown_module(module):
    clean_temporary_pub()


def test_empty_site(pub):
    resp = get_app(pub).get('/backoffice/')
    resp = resp.click('Users', index=0)
    resp = resp.click('New User')
    resp = get_app(pub).get('/backoffice/')
    resp = resp.click('Settings', index=0)


def test_empty_site_but_idp_settings(pub):
    pub.cfg['idp'] = {'xxx': {}}
    pub.write_cfg()
    resp = get_app(pub).get('/backoffice/')
    assert resp.location == 'http://example.net/login/?next=http%3A%2F%2Fexample.net%2Fbackoffice%2F'


def test_with_user(pub):
    create_superuser(pub)
    resp = get_app(pub).get('/backoffice/', status=302)
    assert resp.location == 'http://example.net/login/?next=http%3A%2F%2Fexample.net%2Fbackoffice%2F'


def test_with_superuser(pub):
    create_superuser(pub)
    app = login(get_app(pub))
    app.get('/backoffice/')


def test_admin_redirect(pub):
    create_superuser(pub)
    app = login(get_app(pub))
    assert app.get('/admin/whatever', status=302).location == 'http://example.net/backoffice/whatever'


def test_admin_for_all(pub):
    user = create_superuser(pub)
    role = create_role(pub)

    try:
        open(os.path.join(pub.app_dir, 'ADMIN_FOR_ALL'), 'w').close()
        resp = get_app(pub).get('/backoffice/', status=200)
        # check there is a CSS class
        assert resp.pyquery.find('body.admin-for-all')
        # check there are menu items
        resp.click('Management', index=0)
        resp.click('Forms', index=0)
        resp.click('Settings', index=0)

        # cheeck it's possible to get inside the subdirectories
        resp = get_app(pub).get('/backoffice/settings/', status=200)

        pub.cfg['admin-permissions'] = {'settings': [role.id]}
        pub.write_cfg()
        resp = get_app(pub).get('/backoffice/settings/', status=200)

        # check it doesn't work with a non-empty ADMIN_FOR_ALL file
        fd = open(os.path.join(pub.app_dir, 'ADMIN_FOR_ALL'), 'w')
        fd.write('x.x.x.x')
        fd.close()
        resp = get_app(pub).get('/backoffice/settings/', status=302)

        # check it works if the file contains the user IP address
        fd = open(os.path.join(pub.app_dir, 'ADMIN_FOR_ALL'), 'w')
        fd.write('127.0.0.1')
        fd.close()
        resp = get_app(pub).get('/backoffice/settings/', status=200)

        # check it's also ok if the user is logged in but doesn't have the
        # permissions
        user.is_admin = False
        user.store()
        resp = login(get_app(pub)).get('/backoffice/settings/', status=200)
        # check there are menu items
        resp.click('Management', index=0)
        resp.click('Forms', index=0)
        resp.click('Settings', index=0)

    finally:
        if 'admin-permissions' in pub.cfg:
            del pub.cfg['admin-permissions']
            pub.write_cfg()
        os.unlink(os.path.join(pub.app_dir, 'ADMIN_FOR_ALL'))
        role.remove_self()
        user.is_admin = True
        user.store()


def test_users_roles_menu_entries(pub):
    create_superuser(pub)
    app = login(get_app(pub))
    resp = app.get('/backoffice/')
    assert 'Users' in resp.text
    assert 'Roles' in resp.text
    resp = app.get('/backoffice/menu.json')
    assert 'Users' in [x['label'] for x in resp.json]
    assert 'Roles' in [x['label'] for x in resp.json]

    # don't include users/roles in menu if roles are managed by an external
    # identity provider.
    pub.cfg['sp'] = {'idp-manage-roles': True}
    pub.write_cfg()

    resp = app.get('/backoffice/')
    assert 'Users' not in resp.text
    assert 'Roles' not in resp.text
    resp = app.get('/backoffice/menu.json')
    assert 'Users' not in [x['label'] for x in resp.json]
    assert 'Roles' not in [x['label'] for x in resp.json]


def test_studio_home(pub):
    create_superuser(pub)
    app = login(get_app(pub))
    resp = app.get('/backoffice/')
    assert 'studio' in resp.text
    resp = app.get('/backoffice/studio/')
    assert '../forms/' in resp.text
    assert '../cards/' in resp.text
    assert '../workflows/' in resp.text
    assert 'Logged Errors' in resp.text

    pub.cfg['admin-permissions'] = {}
    for part in ('forms', 'cards', 'workflows'):
        # check section link are not displayed if user has no access right
        pub.cfg['admin-permissions'].update({part: ['x']})  # block access
        pub.write_cfg()
        if part != 'workflows':
            resp = app.get('/backoffice/studio/')
            assert '../%s/' % part not in resp.text
        else:
            resp = app.get('/backoffice/studio/', status=403)  # totally closed

    resp = app.get('/backoffice/')
    assert 'studio' not in resp.text


def test_studio_workflows(pub):
    create_superuser(pub)
    app = login(get_app(pub))
    resp = app.get('/backoffice/workflows/')
    resp = resp.click(r'Default \(cards\)')
    assert 'status/recorded/' in resp.text
    assert 'status/deleted/' in resp.text
    assert 'This is the default workflow,' in resp.text
